@@ -2,12 +2,12 @@
  * SwamiLog.c - Message logging and debugging functions
  *
  * Swami
- * Copyright (C) 1999-2003 Josh Green <jgreen@users.sourceforge.net>
+ * Copyright (C) 1999-2014 Element Green <element@elementsofsound.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * as published by the Free Software Foundation; version 2
+ * of the License only.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,6 +28,29 @@
 #include <glib.h>
 #include <stdarg.h>
 
+GQuark
+swami_error_quark (void)
+{
+  static GQuark q = 0;
+
+  if (q == 0)
+    q = g_quark_from_static_string ("libswami-error-quark");
+
+  return (q);
+}
+
+int
+_swami_ret_g_log (const gchar *log_domain, GLogLevelFlags log_level,
+		  const gchar *format, ...)
+{
+  va_list args;
+  va_start (args, format);
+  g_logv (log_domain, log_level, format, args);
+  va_end (args);
+
+  return (TRUE);
+}
+
 void _swami_pretty_log_handler (GLogLevelFlags level,
 				char *file, char *function, int line,
 				char *format, ...)
@@ -42,6 +65,6 @@ void _swami_pretty_log_handler (GLogLevelFlags level,
   s2 = g_strdup_printf ("file %s: line %d (%s): %s", file, line, function, s);
   g_free (s);
 
-  g_log (NULL, level, s2);
+  g_log (NULL, level, "%s", s2);
   g_free (s2);
 }
