@@ -2,7 +2,7 @@
  * SwamiPlugin.c - Swami plugin system
  *
  * Swami
- * Copyright (C) 1999-2010 Joshua "Element" Green <jgreen@users.sourceforge.net>
+ * Copyright (C) 1999-2014 Element Green <element@elementsofsound.org>
  *
  * Inspired by gstplugin from GStreamer (although re-coded)
  *
@@ -21,16 +21,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA or point your web browser to http://www.gnu.org.
  */
-#include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <dirent.h>
-#include <unistd.h>
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <libinstpatch/misc.h>	/* ipatch_gerror_message() */
 
@@ -39,6 +30,10 @@
 #include "version.h"
 #include "i18n.h"
 
+#ifdef __APPLE__
+#include <CoreFoundation.h>
+#endif
+
 enum
 {
   PROP_0,
@@ -75,12 +70,32 @@ static gboolean swami_plugin_load_recurse (char *file, char *name);
 void
 _swami_plugin_initialize (void)
 {
-#ifdef SWAMI_DEVELOPER	    /* use build tree for loading plugins? */
-  plugin_paths = g_list_append (plugin_paths, BUILD_DIR "/src/plugins");
-  plugin_paths = g_list_append (plugin_paths, BUILD_DIR "/src/plugins/.libs");
-#else  /* ?: no, add the main (installed) library path */
-  plugin_paths = g_list_append (plugin_paths, PLUGINS_DIR);
+  gchar *path = NULL;
+
+#if defined (__APPLE__)
+  CFURLRef ResDirURL;
+  gchar buf[PATH_MAX];
+
+  /* Use plugins within the bundle if we are in an app bundle */
+  ResDirURL = CFBundleCopyBuiltInPlugInsURL (CFBundleGetMainBundle());
+  if (CFURLGetFileSystemRepresentation (ResDirURL, TRUE, (UInt8 *)buf, PATH_MAX)
+      && g_str_has_suffix (buf, ".app/Contents/PlugIns"))
+    path = g_strdup (buf);
+  CFRelease (ResDirURL);
+#elif defined (G_OS_WIN32)
+  gchar *appdir;
+
+  appdir = g_win32_get_package_installation_directory_of_module (NULL);
+  if (appdir)
+  {
+    path = g_build_filename (appdir, "plugins", NULL);  /* !! never freed */
+    g_free (appdir);    /* -- free appdir */
+  }
 #endif
+  if (path)
+    plugin_paths = g_list_append (plugin_paths, path);
+  else
+    plugin_paths = g_list_append (plugin_paths, PLUGINS_DIR);
 }
 
 GType
@@ -366,42 +381,42 @@ swami_plugin_load_all (void)
 
   while (path)
     {
-      g_message (_("Loading plugins from %s"), (char *)path->data);
+      /* g_message (_("Loading plugins from %s"), (char *)path->data); */
       swami_plugin_load_recurse (path->data, NULL);
       path = g_list_next (path);
     }
 
-  g_message (_("Loaded %d plugins"), plugin_seqno);
+  /* g_message (_("Loaded %d plugins"), plugin_seqno); */
 }
 
 static gboolean
 swami_plugin_load_recurse (char *file, char *name)
 {
-  DIR *dir;
-  struct dirent *dirent;
+  GDir *dir;
+  const gchar *d_name;
   gboolean loaded = FALSE;
   char *dirname, *temp;
 
-  dir = opendir (file);
+  dir = g_dir_open (file, 0, NULL);
   if (dir)			/* is "file" a directory or file? */
     {
-      while ((dirent = readdir (dir))) /* its a directory, open it */
+      while ((d_name = g_dir_read_name (dir))) /* its a directory, open it */
 	{
 	  /* don't want to recurse in place or backwards */
-	  if (strcmp (dirent->d_name, ".") && strcmp (dirent->d_name, ".."))
+	  if (strcmp (d_name, ".") && strcmp (d_name, ".."))
 	    {
 	      dirname = g_strjoin (G_DIR_SEPARATOR_S, file,
-				   dirent->d_name, NULL);
+				   d_name, NULL);
 	      loaded = swami_plugin_load_recurse (dirname, name);
 	      g_free (dirname);
 	      if (loaded && name) /* done searching for a specific plugin? */
 		{
-		  closedir (dir);
+		  g_dir_close (dir);
 		  return TRUE;
 		}
 	    }
 	}
-      closedir (dir);
+      g_dir_close (dir);
     }
   else
     { /* search for shared library extension */
