@@ -2,7 +2,7 @@
  * fftune_gui.h - FFTune GUI object (Fast Fourier sample tuning GUI)
  *
  * Swami
- * Copyright (C) 1999-2010 Joshua "Element" Green <jgreen@users.sourceforge.net>
+ * Copyright (C) 1999-2014 Element Green <element@elementsofsound.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -75,6 +75,25 @@ enum
   COL_COUNT	/* count of columns */
 };
 
+typedef struct
+{
+  int limit;
+  char *label;
+} LimitInfo;
+
+/* Default index in limits[] array below */
+#define DEFAULT_LIMIT_INDEX 2
+
+/* Items in limits GtkComboBox */
+const LimitInfo limits[] =
+{
+  {          0, N_("Unlimited") },
+  { 128 * 1024, N_("128K") },
+  {  64 * 1024, N_("64K") },
+  {  32 * 1024, N_("32K") },
+  {  16 * 1024, N_("16K") }
+};
+
 static gboolean plugin_fftune_gui_init (SwamiPlugin *plugin, GError **err);
 static void fftune_gui_panel_iface_init (SwamiguiPanelIface *panel_iface);
 static gboolean fftune_gui_panel_iface_check_selection (IpatchList *selection,
@@ -97,6 +116,9 @@ static void fftune_gui_cb_tunings_change (FFTuneSpectra *spectra, guint count,
 					  gpointer user_data);
 static void fftune_gui_cb_mode_menu_changed (GtkComboBox *combo,
 					     gpointer user_data);
+static void fftune_gui_cb_limit_changed (GtkComboBox *combo, gpointer user_data);
+static void fftune_gui_cb_window_toggled (GtkToggleButton *button,
+                                          gpointer user_data);
 static void fftune_gui_cb_canvas_size_allocate (GtkWidget *widget,
 						GtkAllocation *allocation,
 						gpointer user_data);
@@ -112,6 +134,33 @@ static void fftune_gui_zoom_ofs (FFTuneGui *fftunegui, double zoom_amt,
 				 int zoom_xpos);
 static void fftune_gui_scroll_ofs (FFTuneGui *fftunegui, int index_ofs);
 
+#ifdef _WIN32
+/*
+ * Parse the first float found in a char array. Parsing is terminated
+ * when the end of string is reach or a non digit caracter is encountered.
+ * @param str char array to parse
+ * @return the first float detected in str or 0.0f.
+ * Example: 
+ * "123.2"     -> 123.2f
+ * "123.2text" -> 123.2f
+ * "text123.2" ->   0.0f
+ */
+static float 
+strtof(const char* str, char **endptr)
+{
+    return (float)strtod(str, endptr);
+}
+
+/*
+ return the integer nearest num float value
+*/
+static int
+roundf(float num) 
+{ 
+    return (int)(num < 0 ? num - 0.5 : num + 0.5); 
+}
+#endif
+
 
 /* set plugin information */
 SWAMI_PLUGIN_INFO (plugin_fftune_gui_init, NULL);
@@ -133,8 +182,8 @@ plugin_fftune_gui_init (SwamiPlugin *plugin, GError **err)
   g_object_set (plugin,
 		"name", "FFTuneGui",
 		"version", "1.0",
-		"author", "Josh Green",
-		"copyright", "Copyright (C) 2005",
+		"author", "Element Green",
+		"copyright", "Copyright (C) 2005-2014",
 		"descr", N_("GUI for Fast Fourier Transform sample tuner"),
 		"license", "GPL",
 		NULL);
@@ -292,6 +341,7 @@ fftune_gui_init (FFTuneGui *fftunegui)
   GtkStyle *style;
   GtkAdjustment *adj;
   GtkWidget *widg;
+  int i;
 
   fftunegui->snap_active = FALSE;
   fftunegui->snap_timeout_handler = 0;
@@ -300,7 +350,10 @@ fftune_gui_init (FFTuneGui *fftunegui)
   fftunegui->snap_line_color = 0xFF0000FF;
 
   /* create spectrum tuning object */
-  fftunegui->spectra = g_object_new (FFTUNE_TYPE_SPECTRA, NULL);
+  fftunegui->spectra = g_object_new (g_type_from_name ("FFTuneSpectra"), NULL);
+
+  /* Set default FFT limit */
+  g_object_set (fftunegui->spectra, "limit", limits[DEFAULT_LIMIT_INDEX].limit, NULL);
 
   /* connect to spectrum change signal */
   g_signal_connect (fftunegui->spectra, "spectrum-change",
@@ -314,7 +367,7 @@ fftune_gui_init (FFTuneGui *fftunegui)
   box = gtk_hbox_new (FALSE, 4);
   gtk_widget_show (box);
 
-  lbl = gtk_label_new (_("Sample data"));
+  lbl = gtk_label_new (_("Data"));
   gtk_widget_show (lbl);
   gtk_box_pack_start (GTK_BOX (box), lbl, FALSE, FALSE, 0);
 
@@ -329,6 +382,29 @@ fftune_gui_init (FFTuneGui *fftunegui)
   g_signal_connect (fftunegui->mode_menu, "changed",
 		    (GCallback)fftune_gui_cb_mode_menu_changed, fftunegui);
 
+  lbl = gtk_label_new (_("Limit"));
+  gtk_widget_show (lbl);
+  gtk_box_pack_start (GTK_BOX (box), lbl, FALSE, FALSE, 0);
+
+  widg = gtk_combo_box_new_text ();
+  gtk_widget_show (widg);
+  gtk_box_pack_start (GTK_BOX (box), widg, FALSE, FALSE, 0);
+
+  for (i = 0; i < G_N_ELEMENTS (limits); i++)
+    gtk_combo_box_append_text (GTK_COMBO_BOX (widg), _(limits[i].label));
+
+  gtk_combo_box_set_active (GTK_COMBO_BOX (widg), DEFAULT_LIMIT_INDEX);
+
+  g_signal_connect (widg, "changed",
+		    (GCallback)fftune_gui_cb_limit_changed, fftunegui);
+
+  widg = gtk_toggle_button_new_with_label (_("Window"));
+  gtk_widget_show (widg);
+  gtk_box_pack_start (GTK_BOX (box), widg, FALSE, FALSE, 0);
+
+  g_signal_connect (widg, "toggled",
+		    (GCallback)fftune_gui_cb_window_toggled, fftunegui);
+
   lbl = gtk_label_new (_("Root note"));
   gtk_widget_show (lbl);
   gtk_box_pack_start (GTK_BOX (box), lbl, FALSE, FALSE, 0);
@@ -577,9 +653,10 @@ fftune_gui_cb_tunings_change (FFTuneSpectra *spectra, guint count,
 {
   FFTuneGui *fftunegui = FFTUNE_GUI (user_data);
   GtkTreeIter iter;
-  double power, max_power, freq, cents;
+  double power, max_power = 0.0, freq, cents;
   char powerstr[6], freqstr[32], notestr[11], centsstr[16];
-  int i, note;
+  int note;
+  guint i;
 
   gtk_list_store_clear (fftunegui->freq_store);
 
@@ -597,7 +674,7 @@ fftune_gui_cb_tunings_change (FFTuneSpectra *spectra, guint count,
       if (i == 0) max_power = power;	/* first tuning is max power */
 
       cents = ipatch_unit_hertz_to_cents (freq);
-      note = cents / 100.0 + 0.5;
+      note = (int)(cents / 100.0 + 0.5);
       cents -= note * 100;
 
       sprintf (powerstr, "%0.2f", power / max_power);
@@ -634,6 +711,26 @@ fftune_gui_cb_mode_menu_changed (GtkComboBox *combo, gpointer user_data)
     g_object_set (fftunegui->spectra, "sample-mode", active, NULL);
 }
 
+static void
+fftune_gui_cb_limit_changed (GtkComboBox *combo, gpointer user_data)
+{
+  FFTuneGui *fftunegui = FFTUNE_GUI (user_data);
+  int active;
+
+  active = gtk_combo_box_get_active (combo);
+  if (active != -1)
+    g_object_set (fftunegui->spectra, "limit", limits[active].limit, NULL);
+}
+
+static void
+fftune_gui_cb_window_toggled (GtkToggleButton *button, gpointer user_data)
+{
+  FFTuneGui *fftunegui = FFTUNE_GUI (user_data);
+
+  g_object_set (fftunegui->spectra, "enable-window",
+                gtk_toggle_button_get_active (button), NULL);
+}
+
 static void
 fftune_gui_cb_canvas_size_allocate (GtkWidget *widget,
 				    GtkAllocation *allocation,
@@ -721,7 +818,7 @@ fftune_gui_cb_spectrum_canvas_event (GnomeCanvas *canvas,
 
       if (!fftunegui->snap_active) break;
 
-      ofs = motion_event->x - fftunegui->snap_pos;
+      ofs = (int)(motion_event->x - fftunegui->snap_pos);
       val = ABS (ofs);
       if (val > SNAP_TIMEOUT_PIXEL_RANGE) val = SNAP_TIMEOUT_PIXEL_RANGE;
 
@@ -749,7 +846,7 @@ fftune_gui_cb_spectrum_canvas_event (GnomeCanvas *canvas,
 	  if (ofs >= 0) val = SNAP_SCROLL_MIN;
 	  else val = -SNAP_SCROLL_MIN;
 
-	  fftunegui->scroll_amt = zoom * (ofs * SNAP_SCROLL_MULT + val);
+	  fftunegui->scroll_amt = (int)(zoom * (ofs * SNAP_SCROLL_MULT + val));
 	}
       else fftunegui->scroll_active = FALSE;
 
@@ -788,7 +885,7 @@ fftune_gui_cb_spectrum_canvas_event (GnomeCanvas *canvas,
 
       fftunegui->snap_active = TRUE;
 
-      fftunegui->snap_pos = btn_event->x;
+      fftunegui->snap_pos = (int)btn_event->x;
       height = GTK_WIDGET (fftunegui->canvas)->allocation.height;
 
       points = gnome_canvas_points_new (2);
@@ -841,7 +938,7 @@ fftune_gui_cb_spectrum_canvas_event (GnomeCanvas *canvas,
       if (scroll_event->direction == GDK_SCROLL_DOWN)
 	scale = 1 / scale;
 
-      fftune_gui_zoom_ofs (fftunegui, scale, scroll_event->x);
+      fftune_gui_zoom_ofs (fftunegui, scale, (int)scroll_event->x);
 
       fftunegui->last_wheel_dir = scroll_event->direction;
       fftunegui->last_wheel_time = scroll_event->time;
@@ -930,9 +1027,8 @@ static void
 fftune_gui_zoom_ofs (FFTuneGui *fftunegui, double zoom_amt, int zoom_xpos)
 {
   double zoom;
-  guint start;
   guint spectrum_size;
-  int width, index_ofs;
+  int start, width, index_ofs;
 
   g_return_if_fail (FFTUNE_IS_GUI (fftunegui));
 
@@ -943,7 +1039,7 @@ fftune_gui_zoom_ofs (FFTuneGui *fftunegui, double zoom_amt, int zoom_xpos)
 		"width", &width,
 		NULL);
 
-  index_ofs = zoom_xpos * zoom; /* index to zoom xpos */
+  index_ofs = (int)(zoom_xpos * zoom); /* index to zoom xpos */
   zoom *= zoom_amt;
 
   spectrum_size = SWAMIGUI_SPECTRUM_CANVAS (fftunegui->spectrum)->spectrum_size;
@@ -958,13 +1054,13 @@ fftune_gui_zoom_ofs (FFTuneGui *fftunegui, double zoom_amt, int zoom_xpos)
     }
   else
     {
-      index_ofs -= zoom_xpos * zoom; /* subtract new zoom offset */
+      index_ofs -= (int)(zoom_xpos * zoom); /* subtract new zoom offset */
       if (index_ofs < 0 && -index_ofs > start) start = 0;
       else start += index_ofs;
 
       /* make sure spectrum doesn't end in the middle of the display */
       if (start + width * zoom > spectrum_size)
-	start = spectrum_size - width * zoom;
+	start = (int)(spectrum_size - width * zoom);
     }
 
   g_object_set (fftunegui->spectrum,
@@ -994,7 +1090,7 @@ fftune_gui_scroll_ofs (FFTuneGui *fftunegui, int index_ofs)
 
   spectrum_size = SWAMIGUI_SPECTRUM_CANVAS (fftunegui->spectrum)->spectrum_size;
 
-  last_index = spectrum_size - zoom * width;
+  last_index = (int)(spectrum_size - zoom * width);
   if (last_index < 0) return;	/* spectrum too small for current zoom? */
 
   newstart = (int)start_index + index_ofs;
