@@ -2,7 +2,7 @@
  * SwamiguiPiano.c - Piano canvas item
  *
  * Swami
- * Copyright (C) 1999-2010 Joshua "Element" Green <jgreen@users.sourceforge.net>
+ * Copyright (C) 1999-2014 Element Green <element@elementsofsound.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -502,21 +502,21 @@ swamigui_piano_midi_ctrl_callback (SwamiControl *control,
 				  const GValue *value)
 {
   SwamiguiPiano *piano = SWAMIGUI_PIANO (SWAMI_CONTROL_FUNC(control)->user_data);
-  GValueArray *valarray = NULL;
+  GArray *valarray = NULL;
   SwamiMidiEvent *midi;
   int i, count = 1;		/* default for single values */
 
   /* if its multiple values, fetch the value array */
-  if (G_VALUE_TYPE (value) == G_TYPE_VALUE_ARRAY)
+  if (G_VALUE_TYPE (value) == G_TYPE_ARRAY)
     {
       valarray = g_value_get_boxed (value);
-      count = valarray->n_values;
+      count = valarray->len;
     }
 
   i = 0;
   while (i < count)
     {
-      if (valarray) value = g_value_array_get_nth (valarray, i);
+      if (valarray) value = &g_array_index (valarray, GValue, i);
 
       if (G_VALUE_TYPE (value) == SWAMI_TYPE_MIDI_EVENT
 	  && (midi = g_value_get_boxed (value)))
@@ -764,7 +764,7 @@ swamigui_piano_draw_noteon (SwamiguiPiano *piano, int note, int velocity)
 {
   GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (piano);
   KeyInfo *key_info;
-  double pos, w, h, vel;
+  double pos, w, vel;
   int note_ofs;
   gboolean black;
 
@@ -782,7 +782,6 @@ swamigui_piano_draw_noteon (SwamiguiPiano *piano, int note, int velocity)
 
       /* calculate size and velocity position of white key indicator */
       w = piano->key_width_half * PIANO_WHITE_INDICATOR_WIDTH_SCALE;
-      h = piano->world_height * (PIANO_WHITE_INDICATOR_HEIGHT_SCALE / 2.0);
       vel = velocity * piano->white_vel_range / 127.0 + piano->white_vel_ofs;
     }
   else				/* black key */
@@ -795,7 +794,6 @@ swamigui_piano_draw_noteon (SwamiguiPiano *piano, int note, int velocity)
 
       /* calculate size and velocity position of black key indicator */
       w = piano->key_width_half * PIANO_BLACK_INDICATOR_WIDTH_SCALE;
-      h = piano->black_height * (PIANO_BLACK_INDICATOR_HEIGHT_SCALE / 2.0);
       vel = velocity * piano->black_vel_range / 127.0 + piano->black_vel_ofs;
     }
 
@@ -1068,7 +1066,7 @@ swamigui_piano_pos_to_note (SwamiguiPiano *piano, double x, double y,
     return (-1);
 
   /* calculate note */
-  note = x / piano->key_width;
+  note = (int)(x / piano->key_width);
 
   if (note >= piano->key_count) note = piano->key_count - 1;
   mod = note % 12;
@@ -1095,16 +1093,16 @@ swamigui_piano_pos_to_note (SwamiguiPiano *piano, double x, double y,
 	  if (y < piano->black_vel_ofs) *velocity = 1;
 	  else if (y > piano->black_vel_ofs + piano->black_vel_range)
 	    *velocity = 127;
-	  else *velocity = 1.0 + (y - piano->black_vel_ofs)
-		 / (piano->black_vel_range) * 126.0 + 0.5;
+	  else *velocity = (int)(1.0 + (y - piano->black_vel_ofs)
+		 / (piano->black_vel_range) * 126.0 + 0.5);
 	}
       else			/* white key */
 	{
 	  if (y < piano->white_vel_ofs) *velocity = 1;
 	  else if (y > piano->white_vel_ofs + piano->white_vel_range)
 	    *velocity = 127;
-	  else *velocity = 1.0 + (y - piano->white_vel_ofs)
-		 / (piano->white_vel_range) * 126.0 + 0.5;
+	  else *velocity = (int)(1.0 + (y - piano->white_vel_ofs)
+		 / (piano->white_vel_range) * 126.0 + 0.5);
 	}
     }
 
