@@ -2,7 +2,7 @@
  * SwamiguiRoot.c - Swami main GUI object
  *
  * Swami
- * Copyright (C) 1999-2010 Joshua "Element" Green <jgreen@users.sourceforge.net>
+ * Copyright (C) 1999-2014 Element Green <element@elementsofsound.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,6 +33,9 @@
 #include <libswami/version.h>
 #include <libinstpatch/IpatchParamProp.h>
 #include <libinstpatch/IpatchXml.h>
+#ifdef MAC_INTEGRATION
+#include <gtkosxapplication.h>
+#endif
 
 #include "swamigui.h"
 #include "i18n.h"
@@ -51,6 +54,8 @@ enum
   PROP_PIANO_LOWER_KEYS,
   PROP_PIANO_UPPER_KEYS,
   PROP_DEFAULT_PATCH_TYPE,
+  PROP_MIDDLE_EMUL_ENABLE,
+  PROP_MIDDLE_EMUL_MOD,
   PROP_TREE_STORE_LIST,
   PROP_SELECTION_ORIGIN,
   PROP_SELECTION,
@@ -121,6 +126,20 @@ static guint *swamigui_root_parse_piano_keys (const char *str);
 static char *swamigui_root_encode_piano_keys (const guint *keyvals);
 static GtkWidget *sf2_prop_handler (GtkWidget *widg, GObject *obj);
 static void current_date_btn_clicked (GtkButton *button, gpointer user_data);
+static void cat_map_to_tree_store (GtkTreeStore *store,
+                                   const IpatchSLIInstCatMapEntry *catmap,
+                                   GtkTreeIter *parent);
+static void category_changed_cb (GtkComboBox *combo, gpointer user_data);
+static GtkWidget *sli_inst_prop_handler (GtkWidget *widg, GObject *obj);
+
+#ifdef MAC_INTEGRATION
+static void
+osx_accel_map_foreach_lcb(gpointer data, const gchar *accel_path,
+                          guint accel_key, GdkModifierType accel_mods,
+                          gboolean changed);
+static gboolean swamigui_root_cb_should_quit (GtkosxApplication *theApp,
+					      gpointer data);
+#endif
 
 static guint uiroot_signals[LAST_SIGNAL] = { 0 };
 
@@ -157,12 +176,18 @@ swamigui_init (int *argc, char **argv[])
   if (!g_thread_supported ()) g_thread_init (NULL);
 
   gtk_set_locale ();
-  gtk_init (argc, argv);	/* initialize GTK */
+  gtk_init_check (argc, argv);	/* initialize GTK */
 
   /* set the application/program name, so things work right with recent file chooser
    * even when binary name is different (such as lt-swami) */
   g_set_application_name ("swami");
 
+#ifdef MAC_INTEGRATION
+  /* Create singleton app instance at startup, later calls will return this */
+  GtkosxApplication *theApp = g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
+  theApp = NULL; /* we don't use it now, silence compiler warning */
+#endif
+
   swami_init ();
 
   /* install icon parameter property (for assigning icons to effect controls) */
@@ -178,6 +203,7 @@ swamigui_init (int *argc, char **argv[])
   /* set type specific icons */
   ipatch_type_set (IPATCH_TYPE_DLS2, "icon", SWAMIGUI_STOCK_DLS, NULL);
   ipatch_type_set (IPATCH_TYPE_GIG, "icon", SWAMIGUI_STOCK_GIG, NULL);
+  ipatch_type_set (IPATCH_TYPE_SLI, "icon", SWAMIGUI_STOCK_SLI, NULL);
   ipatch_type_set (IPATCH_TYPE_SF2, "icon", SWAMIGUI_STOCK_SOUNDFONT, NULL);
 
   swamigui_util_init ();
@@ -227,6 +253,7 @@ swamigui_init (int *argc, char **argv[])
   swamigui_tree_get_type ();
   swamigui_tree_store_get_type ();
   swamigui_tree_store_patch_get_type ();
+  swamigui_tree_store_config_get_type ();
   swamigui_util_unit_rgba_color_get_type ();
 
   _swamigui_item_menu_actions_init ();
@@ -245,6 +272,10 @@ swamigui_init (int *argc, char **argv[])
   swamigui_register_prop_glade_widg (IPATCH_TYPE_SF2_IZONE, "PropSF2IZone");
   swamigui_register_prop_glade_widg (IPATCH_TYPE_SF2_SAMPLE, "PropSF2Sample");
 
+  swamigui_register_prop_handler (IPATCH_TYPE_SLI_INST, sli_inst_prop_handler);
+  swamigui_register_prop_glade_widg (IPATCH_TYPE_SLI_ZONE, "PropSF2IZone");
+  swamigui_register_prop_glade_widg (IPATCH_TYPE_SLI_SAMPLE, "PropSF2Sample");
+
 #ifdef PYTHON_SUPPORT
   if (!swamigui_disable_python)
   {
@@ -362,6 +393,16 @@ swamigui_root_class_init (SwamiguiRootClass *klass)
 						       "Default patch type",
 						       IPATCH_TYPE_BASE,
 						       G_PARAM_READWRITE));
+  g_object_class_install_property (obj_class, PROP_MIDDLE_EMUL_ENABLE,
+			g_param_spec_boolean ("middle-emul-enable",
+					      _("Middle button emulation enable"),
+					      _("Enable middle mouse button emulation"),
+					      TRUE, G_PARAM_READWRITE));
+  g_object_class_install_property (obj_class, PROP_MIDDLE_EMUL_MOD,
+			g_param_spec_int ("middle-emul-mod",
+					  _("Middle button emulation modifier"),
+					  _("Middle mouse button emulation key modifier"),
+					  0, 4, 0, G_PARAM_READWRITE));
   g_object_class_install_property (obj_class, PROP_TREE_STORE_LIST,
 		g_param_spec_object ("tree-store-list", "Tree store list",
 				     "List of tree stores",
@@ -440,6 +481,12 @@ swamigui_root_set_property (GObject *object, guint property_id,
     case PROP_DEFAULT_PATCH_TYPE:
       root->default_patch_type = g_value_get_gtype (value);
       break;
+    case PROP_MIDDLE_EMUL_ENABLE:
+      root->middle_emul_enable = g_value_get_boolean (value);
+      break;
+    case PROP_MIDDLE_EMUL_MOD:
+      root->middle_emul_mod = g_value_get_int (value);
+      break;
     case PROP_TREE_STORE_LIST:
       if (root->tree_stores) g_object_unref (root->tree_stores);
       root->tree_stores = IPATCH_LIST (g_value_dup_object (value));
@@ -515,16 +562,20 @@ swamigui_root_get_property (GObject *object, guint property_id,
       g_value_set_int (value, root->tips_position);
       break;
     case PROP_PIANO_LOWER_KEYS:
-      g_value_set_string_take_ownership (value, swamigui_root_encode_piano_keys
-					 (root->piano_lower_keys));
+      g_value_take_string (value, swamigui_root_encode_piano_keys (root->piano_lower_keys));
       break;
     case PROP_PIANO_UPPER_KEYS:
-      g_value_set_string_take_ownership (value, swamigui_root_encode_piano_keys
-					 (root->piano_upper_keys));
+      g_value_take_string (value, swamigui_root_encode_piano_keys (root->piano_upper_keys));
       break;
     case PROP_DEFAULT_PATCH_TYPE:
       g_value_set_gtype (value, root->default_patch_type);
       break;
+    case PROP_MIDDLE_EMUL_ENABLE:
+      g_value_set_boolean (value, root->middle_emul_enable);
+      break;
+    case PROP_MIDDLE_EMUL_MOD:
+      g_value_set_int (value, root->middle_emul_mod);
+      break;
     case PROP_TREE_STORE_LIST:
       g_value_set_object (value, root->tree_stores);
       break;
@@ -577,10 +628,10 @@ swamigui_root_init (SwamiguiRoot *root)
 #endif
 
   root->update_interval = 40;
-  root->quit_confirm = SWAMIGUI_QUIT_CONFIRM_NEVER;
+  root->quit_confirm = SWAMIGUI_QUIT_CONFIRM_UNSAVED;
   root->splash_enable = TRUE;
   root->splash_delay = SWAMIGUI_ROOT_DEFAULT_SPLASH_DELAY;
-  root->tips_enable = FALSE;
+  root->tips_enable = TRUE;
   root->default_patch_type = IPATCH_TYPE_SF2;
 
   root->piano_lower_keys
@@ -588,17 +639,24 @@ swamigui_root_init (SwamiguiRoot *root)
   root->piano_upper_keys
     = swamigui_root_parse_piano_keys (SWAMIGUI_ROOT_DEFAULT_UPPER_KEYS);
 
+  root->middle_emul_enable = TRUE;
+
   swami_object_set (G_OBJECT (root),
 		    "name", "Swami",
 		    "flags", SWAMI_OBJECT_SAVE | SWAMI_OBJECT_USER,
 		    NULL);
 
-  /* create tree store */
+  /* create instrument tree store */
   root->patch_store = SWAMIGUI_TREE_STORE
     (swamigui_tree_store_patch_new ()); /* ++ ref */
   swami_object_set (G_OBJECT (root->patch_store), "name", "Patches", NULL);
   swami_root_add_object (SWAMI_ROOT (root), G_OBJECT (root->patch_store));
 
+  /* create config tree store */
+  root->config_store = swamigui_tree_store_config_new (); /* ++ ref */
+  swami_object_set (G_OBJECT (root->config_store), "name", "Config", NULL);
+  swami_root_add_object (SWAMI_ROOT (root), G_OBJECT (root->config_store));
+
   /* create GUI control queue */
   root->ctrl_queue = swami_control_queue_new (); /* ++ ref control queue */
 
@@ -641,16 +699,17 @@ swamigui_root_init (SwamiguiRoot *root)
 			   root->ctrl_queue);
 
   /* connect controls to ipatch event senders */
-  swami_control_connect (swami_patch_prop_title_control,
+  swami_control_connect (swami_patch_get_prop_title_control(),
 			 SWAMI_CONTROL (root->ctrl_prop), 0);
-  swami_control_connect (swami_patch_add_control,
+  swami_control_connect (swami_patch_get_add_control(),
 			 SWAMI_CONTROL (root->ctrl_add), 0);
-  swami_control_connect (swami_patch_remove_control,
+  swami_control_connect (swami_patch_get_remove_control(),
 			 SWAMI_CONTROL (root->ctrl_remove), 0);
 
   /* add patch store to tree store list */
   list = ipatch_list_new ();
   list->items = g_list_append (list->items, g_object_ref (root->patch_store));
+  list->items = g_list_append (list->items, g_object_ref (root->config_store));
   root->tree_stores = list;
 
   /* create the FluidSynth wavetable object */
@@ -661,6 +720,8 @@ swamigui_root_init (SwamiguiRoot *root)
   {
     swami_object_set (root->wavetbl, "name", "FluidSynth1", NULL);
     swami_root_add_object (SWAMI_ROOT (root), G_OBJECT (root->wavetbl));
+    swamigui_tree_store_insert (root->config_store, G_OBJECT (root->wavetbl),
+				NULL, NULL, NULL, 0, NULL);
   }
 }
 
@@ -668,15 +729,23 @@ static void
 swamigui_root_finalize (GObject *object)
 {
   SwamiguiRoot *root = SWAMIGUI_ROOT (object);
+  GList *p;
 
   g_object_unref (root->patch_store);
+  g_object_unref (root->config_store);
   g_object_unref (root->tree_stores);
 
   if (root->selection) g_object_unref (root->selection);
-  if (root->wavetbl) g_object_unref (root->wavetbl);
+
+  gtk_object_destroy (GTK_OBJECT (root->main_window));
+
+  if (root->wavetbl)
+  {
+    swami_wavetbl_close (root->wavetbl);
+    g_object_unref (root->wavetbl);
+  }
 
   if (root->solo_item) g_object_unref (root->solo_item);
-  if (root->solo_item_icon) g_free (root->solo_item_icon);
 
   g_object_unref (root->ctrl_queue);
 
@@ -687,9 +756,16 @@ swamigui_root_finalize (GObject *object)
   g_object_unref (root->ctrl_add);
   g_object_unref (root->ctrl_remove);
 
+  g_free (root->piano_lower_keys);
+  g_free (root->piano_upper_keys);
+
   if (root->loaded_xml_config) ipatch_xml_destroy (root->loaded_xml_config);
 
-  if (parent_class)
+  // Free widgets in panel cache
+  for (p = root->panel_cache; p; p = g_list_delete_link (p, p))
+    g_object_unref (p->data);
+
+  if (parent_class->finalize)
     parent_class->finalize (object);
 }
 
@@ -866,10 +942,8 @@ swamigui_root_quit (SwamiguiRoot *root)
 				  GTK_MESSAGE_QUESTION,
 				  GTK_BUTTONS_NONE, "%s", s);
   gtk_dialog_add_buttons (GTK_DIALOG (popup),
-			  GTK_STOCK_CANCEL,
-			  GTK_RESPONSE_CANCEL,
-			  GTK_STOCK_QUIT,
-			  GTK_RESPONSE_OK,
+			  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			  GTK_STOCK_QUIT, GTK_RESPONSE_OK,
 			  NULL);
   g_signal_connect (popup, "response",
 		    G_CALLBACK (swamigui_cb_quit_response), root);
@@ -1037,7 +1111,6 @@ swamigui_root_save_prefs (SwamiguiRoot *root)
 gboolean
 swamigui_root_load_prefs (SwamiguiRoot *root)
 {
-  IpatchXmlNode *xmlnode;
   GNode *xmltree, *n;
   char *pref_filename;
   GParamSpec *pspec;
@@ -1079,7 +1152,6 @@ swamigui_root_load_prefs (SwamiguiRoot *root)
 
   for (n = xmltree->children; n; n = n->next)
   {
-    xmlnode = (IpatchXmlNode *)(n->data);
     name = ipatch_xml_get_name (n);
     if (!name) continue;
 
@@ -1159,6 +1231,17 @@ swamigui_get_root (gpointer gobject)
   return (root);
 }
 
+/*
+ * Getter returning swamigui_root.
+ * Useful when libswamigui is used as a shared library
+   For example, plugins fluidsynth_gui and fluidsynth_plugin need swamigui_root.
+ */
+SwamiguiRoot *
+swamigui_get_swamigui_root (void)
+{
+	return (swamigui_root);
+}
+
 /* Create main window and controls */
 static void
 swamigui_root_create_main_window (SwamiguiRoot *root)
@@ -1186,6 +1269,12 @@ swamigui_root_create_main_window (SwamiguiRoot *root)
 
   g_signal_connect (root->main_window, "delete_event",
 		    G_CALLBACK (swamigui_root_cb_main_window_delete), root);
+#ifdef MAC_INTEGRATION
+  GtkosxApplication *theApp = g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
+
+  g_signal_connect (theApp, "NSApplicationBlockTermination",
+		    G_CALLBACK (swamigui_root_cb_should_quit), root);
+#endif
 
   /* add the item menu keyboard accelerators */
   gtk_window_add_accel_group (GTK_WINDOW (root->main_window),
@@ -1204,8 +1293,33 @@ swamigui_root_create_main_window (SwamiguiRoot *root)
   gtk_paned_pack1 (GTK_PANED (hpaned), tempbox, TRUE, TRUE);
 
   widg = swamigui_menu_new ();
+#ifndef MAC_INTEGRATION
   gtk_widget_show (widg);
   gtk_box_pack_start (GTK_BOX (tempbox), widg, FALSE, FALSE, 0);
+#else /* menu is handled a bit differently on OS X */
+  GtkUIManager *mgr = (SWAMIGUI_MENU (widg))->ui;
+  GtkWidget *mitem;
+
+  /* Set the global menu bar to the main menu */
+  mitem = gtk_ui_manager_get_widget(mgr, "/MenuBar");
+  gtkosx_application_set_menu_bar(theApp, GTK_MENU_SHELL (mitem));
+  /* Move Help/About and Edit/Preferences menu items to the app menu */
+  mitem = gtk_ui_manager_get_widget (mgr, "/MenuBar/HelpMenu/About");
+  gtkosx_application_insert_app_menu_item (theApp, mitem, 0);
+  mitem = gtk_separator_menu_item_new ();
+  gtkosx_application_insert_app_menu_item (theApp, mitem, 1);
+  mitem = gtk_ui_manager_get_widget (mgr, "/MenuBar/EditMenu/Preferences");
+  gtkosx_application_insert_app_menu_item (theApp, mitem, 2);
+  /* Hide Quit menu item which is added automatically to the app menu */
+  mitem = gtk_ui_manager_get_widget (mgr, "/MenuBar/FileMenu/Quit");
+  gtk_widget_hide(mitem);
+  /* Designate Help menu and create Window menu */
+  mitem = gtk_ui_manager_get_widget (mgr, "/MenuBar/HelpMenu");
+  gtkosx_application_set_help_menu (theApp, GTK_MENU_ITEM (mitem));
+  gtkosx_application_set_window_menu (theApp, NULL);
+  /* Convert menu accelerator key modifiers */
+  gtk_accel_map_foreach (NULL, osx_accel_map_foreach_lcb);
+#endif
 
   root->tree = swamigui_tree_new (root->tree_stores);
   gtk_widget_show (root->tree);
@@ -1248,7 +1362,7 @@ swamigui_root_create_main_window (SwamiguiRoot *root)
   swami_control_connect (rootsel_ctrl, ctrl, SWAMI_CONTROL_CONN_BIDIR);
   g_object_unref (ctrl);	/* -- unref tree control */
 
-  root->panel_selector = swamigui_panel_selector_new ();
+  root->panel_selector = swamigui_panel_selector_new (root);
   gtk_widget_show (root->panel_selector);
   gtk_paned_pack2 (GTK_PANED (vpaned), root->panel_selector, FALSE, TRUE);
 
@@ -1346,6 +1460,10 @@ swamigui_root_create_main_window (SwamiguiRoot *root)
   g_object_unref (midihub);	/* -- unref creator's reference */
 
   gtk_widget_show (root->main_window);
+#ifdef MAC_INTEGRATION
+  /* Tell OS X that we are ready launching now */
+  gtkosx_application_ready (theApp);
+#endif
 }
 
 /* Callback for SwamiguiRoot::selection-single to update Wavetbl solo-item if enabled */
@@ -1376,10 +1494,7 @@ swamigui_root_update_solo_item (SwamiguiRoot *root, GObject *solo_item)
     ipatch_type_get (G_OBJECT_TYPE (solo_item), "category", &category, NULL);
 
     if (category != IPATCH_CATEGORY_SAMPLE_REF && category != IPATCH_CATEGORY_INSTRUMENT_REF)
-    {
-      g_object_unref (solo_item);       /* -- unref solo item */
       solo_item = NULL;
-    }
   }
 
   /* If previous solo item, restore its icon */
@@ -1387,18 +1502,18 @@ swamigui_root_update_solo_item (SwamiguiRoot *root, GObject *solo_item)
   {
     swamigui_tree_store_change (root->patch_store, root->solo_item, NULL,
                                 root->solo_item_icon);
-    g_object_unref (root->solo_item);
+    g_object_unref (root->solo_item);   // -- unref solo item
     root->solo_item = NULL;
-    root->solo_item_icon = NULL;
+    root->solo_item_icon = NULL;        // !! solo_item_icon is a static string pointer, do not free
   }
 
   if (solo_item)
   {
     if (swamigui_tree_store_item_get_node (root->patch_store, solo_item, &iter))
     {
-      root->solo_item = g_object_ref (solo_item);
+      root->solo_item = g_object_ref (solo_item);               // ++ ref solo item
 
-      /* !! icon column is a static string - not allocated */
+      /* !! icon column is a static string pointer - not allocated */
       gtk_tree_model_get (GTK_TREE_MODEL (root->patch_store), &iter,
                           SWAMIGUI_TREE_STORE_ICON_COLUMN, &root->solo_item_icon,
                           -1);
@@ -1553,3 +1668,181 @@ current_date_btn_clicked (GtkButton *button, gpointer user_data)
 
   gtk_entry_set_text (GTK_ENTRY (user_data), datestr);
 }
+
+/* handler for IpatchSLIInst glade property interface.
+ * Creates the Category combo box and selects its initial item */
+static GtkWidget *
+sli_inst_prop_handler (GtkWidget *widg, GObject *obj)
+{
+  GtkWidget *combo;
+  GtkTreeModel *model;
+  GtkCellRenderer *cell;
+  gchar *catpath;
+  GtkTreeIter iter;
+
+  if (!widg) /* create widget? */
+  {
+    /* get combo box widget */
+    widg = swamigui_util_glade_create ("PropSLIInst");
+    combo = swamigui_util_glade_lookup (widg, "ComboCategory");
+    gtk_combo_box_set_wrap_width (GTK_COMBO_BOX (combo), 2);
+    /* create and attach a tree model */
+    /* ++ ref new store */
+    model = GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_STRING));
+    cat_map_to_tree_store (GTK_TREE_STORE (model),
+                           ipatch_sli_inst_cat_map, NULL);
+    gtk_combo_box_set_model (GTK_COMBO_BOX (combo), model);
+    g_object_unref (model); /* -- unref store */
+    /* set cell renderer */
+    cell = gtk_cell_renderer_text_new ();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), cell, FALSE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo), cell, "text", 0);
+    /* connect callback function */
+    g_signal_connect (combo, "changed", G_CALLBACK (category_changed_cb), NULL);
+  } else {
+    combo = swamigui_util_glade_lookup (widg, "ComboCategory");
+  }
+
+  /* connect prop controls to supplied object */
+  swamigui_control_glade_prop_connect (widg, obj);
+  /* FIXME: setting object data may be a hack due to the lack of understanding
+   * if a better way exists to get this in the callback function */
+  g_object_set_data (G_OBJECT (combo), "controlled-object", obj);
+
+  /* set combo box to match category of the object */
+  /* (setting active item in combo box emits a changed signal) */
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+  catpath = ipatch_sli_inst_get_category_as_path (IPATCH_SLI_INST (obj));
+  if (gtk_tree_model_get_iter_from_string (model, &iter, catpath))
+    gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
+  g_free (catpath);
+
+  return (widg);
+}
+
+static void
+cat_map_to_tree_store (GtkTreeStore *store, const IpatchSLIInstCatMapEntry *catmap, GtkTreeIter *parent)
+{
+  GtkTreeIter child;
+  guint i;
+  gchar *label, *prefix;
+
+  g_return_if_fail (store != NULL && catmap != NULL);
+
+  for (i = 0; catmap[i].code != '@'; i++)
+  {
+    if (parent)
+    {
+      gtk_tree_model_get (GTK_TREE_MODEL (store), parent, 0, &prefix, -1);
+      label = g_strjoin (" | ", prefix,
+                         ipatch_sli_inst_cat_strings[catmap[i].name_idx], NULL);
+      g_free (prefix);
+    } else
+      label = g_strdup (ipatch_sli_inst_cat_strings[catmap[i].name_idx]);
+    gtk_tree_store_append (store, &child, parent);
+    gtk_tree_store_set (store, &child, 0, label, -1);
+    g_free (label);
+    if (catmap[i].submap) /* recursion for subcat maps */
+      cat_map_to_tree_store (store, catmap[i].submap, &child);
+  }
+  /* Add Other category only at the topmost level */
+  if (!parent)
+  {
+    gtk_tree_store_append (store, &child, parent);
+    gtk_tree_store_set (store, &child, 0,
+                        ipatch_sli_inst_cat_strings[catmap[i].name_idx], -1);
+  }
+}
+
+/* callback when Category combo box is changed */
+static void
+category_changed_cb (GtkComboBox *combo, gpointer user_data)
+{
+  const IpatchSLIInstCatMapEntry *catmap = ipatch_sli_inst_cat_map;
+  IpatchSLIInst *inst;
+  gboolean ret;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  GtkTreePath *path;
+  gint *pinds;
+  guint old_cat, new_cat, i, d;
+
+  inst = g_object_get_data (G_OBJECT (combo), "controlled-object");
+  g_return_if_fail (IPATCH_IS_SLI_INST (inst));
+
+  model = gtk_combo_box_get_model (combo);
+  ret = gtk_combo_box_get_active_iter (combo, &iter);
+  g_return_if_fail (ret == TRUE);
+  path = gtk_tree_model_get_path (model, &iter); /* ++ ref path */
+  pinds = gtk_tree_path_get_indices (path);
+  d = gtk_tree_path_get_depth (path);
+
+  for (new_cat = 0, i = 0; catmap && i < d; i++)
+  {
+    new_cat <<= 8;
+    new_cat |= catmap[pinds[i]].code;
+    catmap = catmap[pinds[i]].submap;
+  }
+  if (i < 2) /* treat subcat as Other if only main category is selected */
+  {
+    new_cat <<= 8;
+    new_cat |= '@';
+  }    
+  gtk_tree_path_free (path); /* -- unref path */
+
+  g_object_get (inst, "category", &old_cat, NULL);
+  if (new_cat != old_cat)
+    g_object_set (inst, "category", new_cat, NULL);
+}
+
+#ifdef MAC_INTEGRATION
+/* Callback to convert Ctrl to Command in menu accelerator keys */
+static void
+osx_accel_map_foreach_lcb(gpointer data, const gchar *accel_path,
+                          guint accel_key, GdkModifierType accel_mods,
+                          gboolean changed)
+{
+  if (accel_mods & GDK_CONTROL_MASK)
+  { 
+    accel_mods &= ~GDK_CONTROL_MASK; 
+    accel_mods |= GDK_META_MASK;
+
+    gtk_accel_map_change_entry(accel_path, accel_key, accel_mods, FALSE);
+  } 
+}
+
+/* Callback called for NSApplicationShouldTerminate notification
+ * we return TRUE to veto the termination here because the user can
+ * cancel the quit dialog */
+static gboolean swamigui_root_cb_should_quit (GtkosxApplication *theApp,
+					      gpointer data)
+{
+  swamigui_root_quit (SWAMIGUI_ROOT (data));
+  return TRUE;
+}
+#endif
+
+/**
+ * swamigui_root_is_middle_click:
+ * @root: Root object or NULL to use global root object
+ * @event: Button event
+ *
+ * Check if a button event is a middle click, taking into account middle button emulation settings.
+ */
+gboolean
+swamigui_root_is_middle_click (SwamiguiRoot *root, GdkEventButton *event)
+{
+  g_return_val_if_fail (!root || SWAMIGUI_IS_ROOT (root), FALSE);
+  g_return_val_if_fail (event != NULL, FALSE);
+
+  if (event->type == GDK_BUTTON_PRESS && event->button == 2)
+    return (TRUE);
+
+  if (!root) root = swamigui_root;
+  if (!root) return (FALSE);
+
+  // Middle button emulation enabled, left mouse button click and modifier held?
+  return (event->type == GDK_BUTTON_PRESS && event->button == 1
+          && root->middle_emul_enable && (event->state & (GDK_MOD1_MASK << root->middle_emul_mod)));
+}
+
